/*

   timer.c

   Reentrant versions of times and gettimeofday system calls for the
   clock and time ANSI C routines.

   This is for openVMS with EGCS 1.1 (based on GNU C 2.8)

   Copyright  by Klaus Kmpf (kkaempf@rmi.de) 1998

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that: (1) source code distributions
   retain the above copyright notice and this paragraph in its entirety, (2)
   distributions including binary code include the above copyright notice and
   this paragraph in its entirety in the documentation or other materials
   provided with the distribution, and (3) all advertising materials mentioning
   features or use of this software display the following acknowledgement:
   ``This product includes software developed by Klaus Kmpf, Nrnberg,
   Germany.''
   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  */

#include <reent.h>
#include <time.h>
#include <sys/time.h>
#include <sys/times.h>
#include <_syslist.h>
#include <errno.h>

/* We use the errno variable used by the system dependent layer.  */
#undef errno
extern int errno;

#include <sys$routines.h>
#include <jpidef.h>
#include "jpilib.h"
#include "timeconv.h"

/*
FUNCTION
	<<_times_r>>---Reentrant version of times
	
INDEX
	_times_r

ANSI_SYNOPSIS
	#include <reent.h>
	#include <sys/times.h>
	clock_t _times_r(struct _reent *<[ptr]>, struct tms *<[ptms]>);

TRAD_SYNOPSIS
	#include <reent.h>
	#include <sys/times.h>
	clock_t _times_r(<[ptr]>, <[ptms]>)
	struct _reent *<[ptr]>;
	struct tms *<[ptms]>;

DESCRIPTION
	This is a reentrant version of <<times>>.  It
	takes a pointer to the global data block, which holds
	<<errno>>.
*/

clock_t
_times_r (ptr, ptms)
     struct _reent *ptr;
     struct tms *ptms;
{
  struct timeval timeval;

  ptms->tms_utime = jpi_long_r (ptr, JPI$_CPUTIM);	/* User CPU time  */
  ptms->tms_stime = jpi_long_r (ptr, JPI$_LOGINTIM);	/* System CPU time  */
  ptms->tms_cutime = 0;				/* childs user cpu time  */
  ptms->tms_cstime = 0;				/* childs system cpu time  */

  _gettimeofday_r (ptr, &timeval, (struct timezone *)0);

  return (clock_t)(timeval.tv_sec);
}


/*
FUNCTION
	<<gettimeofday>>---Reentrant version of gettimeofday
	
INDEX
	gettimeofday

ANSI_SYNOPSIS
	#include <time.h>
	int gettimeofday(struct timeval *<[ptimeval]>,
		struct timezone *<[ptimezone]>);

TRAD_SYNOPSIS
	#include <time.h>
	int gettimeofday(<[ptimeval]>, <[ptimezone]>)
	struct timeval *<[ptimeval]>;
	struct timezone *<[ptimezone]>;

DESCRIPTION
	This is <<gettimeofday>>.

	This function is only available for a few targets.
	Check libc.a to see if its available on yours.
*/

int
gettimeofday (ptimeval, ptimezone)
     struct timeval *ptimeval;
     struct timezone *ptimezone;
{
  return (_gettimeofday_r (_REENT, ptimeval, ptimezone));
}


/*
FUNCTION
	<<_gettimeofday_r>>---Reentrant version of gettimeofday

INDEX
	_gettimeofday_r

ANSI_SYNOPSIS
	#include <reent.h>
	#include <time.h>
	int _gettimeofday_r(struct _reent *<[ptr]>,
		struct timeval *<[ptimeval]>,
		struct timezone *<[ptimezone]>);

TRAD_SYNOPSIS
	#include <reent.h>
	#include <time.h>
	int _gettimeofday_r(<[ptr]>, <[ptimeval]>, <[ptimezone]>)
	struct _reent *<[ptr]>;
	struct timeval *<[ptimeval]>;
	struct timezone *<[ptimezone]>;

DESCRIPTION
	This is a reentrant version of <<gettimeofday>>.  It
	takes a pointer to the global data block, which holds
	<<errno>>.

	This function is only available for a few targets.
	Check libc.a to see if its available on yours.
*/

int
_gettimeofday_r (ptr, ptimeval, ptimezone)
     struct _reent *ptr;
     struct timeval *ptimeval;
     struct timezone *ptimezone;
{
  long long vmst;

  if (ptimeval == 0)
    {
      ptr->_errno = EINVAL;
      return -1;
    }

  SYS$GETTIM (&vmst);
  ptimeval->tv_sec = _vmst2unixt (vmst, &ptimeval->tv_usec);

  return 0;
}
