#define DEBUG 1
/*
   unixio.c

   unix compatible i/o

   This is for openVMS with EGCS 1.1 (based on GNU C 2.8)

   Copyright  by Klaus Kmpf (kkaempf@rmi.de) 1998

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that: (1) source code distributions
   retain the above copyright notice and this paragraph in its entirety, (2)
   distributions including binary code include the above copyright notice and
   this paragraph in its entirety in the documentation or other materials
   provided with the distribution, and (3) all advertising materials mentioning
   features or use of this software display the following acknowledgement:
   ``This product includes software developed by Klaus Kmpf, Nrnberg,
   Germany.''
   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

 */
 
#define UNIXIO_C

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <stdlib.h>
#include <vms/iodef.h>
#include <vms/ssdef.h>
#include <vms/stsdef.h>
#include <vms/descrip.h>
#include <vms/starlet.h>

#include "rms.h"

#include <errno.h>

#include "unixio.h"

int vms_errno = 0;

#define REC_SIZE 5120
#define BLOCK_SIZE 512

/*--------------------------------------------------------*/

/*
 * local part
 */

#define MAXPHYSBUFSIZE	4096	/* max buffer size in sectors */

#define MAXOPENFILES 256

struct fd_struct {
  struct FAB fab;		/* fab */
  struct RAB rab;		/* rab */
  int mode;			/* from fcntl */
  char *path;			/* name from open() */
  unsigned char *physbuf;	/* buffer ptr, = 0 if unbuffered */
  int bufsize;
};

int __initialize_unixio (void);


static struct fd_struct *fds[MAXOPENFILES];
  
static void free_fd (int fd);
static int alloc_fd (void);
static int init_rabfab (struct RAB *rab, struct FAB *fab, char *path);
static void free_rabfab (struct RAB *rab, struct FAB *fab);

/*--------------------------------------------------------*/

int
__initialize_unixio (void)
{
  int i;

  for (i=0; i<MAXOPENFILES; i++)
    fds[i] = 0;
  vms_errno = 0;

  return 0;
}

static int
alloc_fd (void)
{
  int i;

  for (i=0; i<MAXOPENFILES; i++)
    if (fds[i] == 0)
      break;

  if (i == MAXOPENFILES)
    {
      errno = ENFILE;
      return -1;
    }

  fds[i] = (struct fd_struct *)malloc (sizeof (struct fd_struct));
  if (fds[i] == 0)
    {
      errno = ENOMEM;
      return -1;
    }

  memset ((void *)fds[i], 0, sizeof (struct fd_struct));

  fds[i]->physbuf = (unsigned char *) malloc (MAXPHYSBUFSIZE);
  if (fds[i]->physbuf == 0)
    {
      free_fd (i);
      errno = ENOMEM;
      return -1;
    }
  fds[i]->bufsize = MAXPHYSBUFSIZE;

  return i;
}


static void
free_fd (int fd)
{
  if (fds[fd] == 0)
    {
      return;
    }

  if (fds[fd]->path != 0)
    free (fds[fd]->path);

  free (fds[fd]);
  fds[fd] = 0;

  return;
}


static int
init_rabfab (struct RAB *rab, struct FAB *fab, char *path)
{
  char *name, *ext;

  /* init from default */

  memcpy (fab, &(cc$rms_fab), sizeof (struct FAB));
  memcpy (rab, &(cc$rms_rab), sizeof (struct RAB));

  /* check & set name */

  name = strdup (path);
  path = name;
  ext = strchr (path, ']');
  if (ext)
    path = ext+1;
  ext = strchr (path, '.');
  if (ext)
    {
      *ext++ = 0;
      if (strchr (ext, '.'))
	{
	  free (name);
	  errno = EINVAL;
	  return -1;
	}
    }
  else
    ext = "";

#if DEBUG
fprintf (stderr, "init_rabfab %s.%s\n", name, ext);
#endif

  fab->fab$l_fna = (const char *)name;
  fab->fab$b_fns = strlen (name);
  fab->fab$l_dna = (const char *)ext;
  fab->fab$b_dns = strlen (ext);

  /* init rab */

  rab->rab$l_fab = fab;

  return 0;
}


static void
free_rabfab (struct RAB *rab, struct FAB *fab)
{
  if (fab->fab$l_fna)
    free ((char *)fab->fab$l_fna);
  rab->rab$l_fab = 0;

  return;
}


#if DEBUG
static void
showhex(char *why, unsigned char *what, int len)
{
  fprintf(stderr,"Showhex %s\n", why);
  while (len > 0) {
    fprintf(stderr,"%02x ", *what++);
    len--;
  }
  fprintf(stderr,"\n");
  return;
}
#endif /* DEBUG */

/*-----------------------------------------------------------------*/

int
vms_open (char *path, int mode)
{
  int fd;
  int res;
  struct RAB *rab;
  struct FAB *fab;

#if DEBUG
  fprintf (stderr, "vms_open (%s, %d)\n", path, mode);
#endif

  fd = alloc_fd ();

  if (fd < 0)
    return fd;

  /* get rab/fab pointer */

  rab = &fds[fd]->rab;
  fab = &fds[fd]->fab;

  if (init_rabfab (rab, fab, path) < 0)
    {
      free_fd (fd);
      return -1;
    }

  rab->rab$l_ubf = fds[fd]->physbuf;
  rab->rab$w_usz = fds[fd]->bufsize;

  if ((mode == O_RDONLY)
     || (mode == O_RDWR))
    fds[fd]->rab.rab$v_rah = 1;
  if ((mode == O_WRONLY)
     || (mode == O_RDWR))
    {
      fds[fd]->fab.fab$v_ctg = 1;
      fds[fd]->fab.fab$v_put = 1;
      fds[fd]->fab.fab$v_nil = 1;
    }

  fds[fd]->mode = mode;
  fds[fd]->path = strdup (path);

  res = sys$open (fab);
#if DEBUG
fprintf (stderr, "sys$open = %x\n", res);
#endif
  if (! res & 1)
    {
      free_rabfab (rab, fab);
      free_fd (fd);
      vms_errno = res;
      errno = ENOENT;
      return -1;
    }

  return fd;
}


int
vms_close (int fd)
{
  struct RAB *rab;
  struct FAB *fab;

#if DEBUG
  fprintf (stderr, "vms_close (%x)\n", fd);
#endif

  if (fds[fd] == 0)
    {
      errno = EINVAL;
      return -1;
    }

  /* get rab/fab pointer */

  rab = &fds[fd]->rab;
  fab = &fds[fd]->fab;

  free_rabfab (rab, fab);

  if (fds[fd]->physbuf != 0)
    free (fds[fd]->physbuf);

  free_fd (fd);

  return;
}


int
vms_stat (char *path, struct stat *st)
{
#if DEBUG
  fprintf (stderr, "vms_stat (%d, %p)\n", path, st);
#endif
  return -1;
}

int
vms_read (int fd, char *buf, unsigned int nbyte)
{
  unsigned char *ptr;
  unsigned int count = 0;
  int bsize;
#if DEBUG
  fprintf (stderr, "vms_read (%x, %p, %d)\n", fd, buf, nbyte);
#endif

  if (fds[fd] == 0)
    {
      errno = EBADF;
      return -1;
    }

  return count;
}

int
vms_write (int fd, char *buf, unsigned int nbyte)
{
  unsigned char *ptr;
  unsigned int count = 0;
  int bsize;
#if DEBUG
fprintf (stderr, "vms_write (%x, %p, %d)\n", fd, buf, nbyte);
#endif

  if (fds[fd] == 0)
    {
      errno = EBADF;
      return -1;
    }

  if (fds[fd]->mode != O_RDWR)
    {
      errno = EROFS;
      return -1;
    }

  return count;
}


int
vms_lseek (int fd, off_t offset, int whence)
{
  int diff;
#if DEBUG
fprintf (stderr, "vms_lseek (%x, %ld, %d)\n", fd, offset, whence);
#endif

  if (fds[fd] == 0)
    {
      errno = EBADF;
      return -1;
    }

  return -1;
}


int
vms_isatty (int fd)
{
  return 0;
}


/* EOF */
